package de.fraunhofer.iem.kpiCalculator.core.strategy

import de.fraunhofer.iem.kpiCalculator.model.kpi.hierarchy.KpiCalculationResult

object MaximumKPICalculationStrategy {
    fun calculateKPI(childScores: List<Pair<KpiCalculationResult, Double>>): KpiCalculationResult {

        if (childScores.isEmpty()) {
            return KpiCalculationResult.Empty
        }

        val successScores = childScores.filterIsInstance<KpiCalculationResult.Success>()

        val max =
            if (successScores.isEmpty())
                0
            else
                successScores.maxOf { it.score }

        if (childScores.size != successScores.size) {
            val failed = childScores.filter { it.first !is KpiCalculationResult.Success }
            val missingWeights = failed.sumOf { it.second }
            return KpiCalculationResult.Incomplete(
                score = max,
                reason = "Missing ${failed.size} elements.",
                missingWeights = missingWeights
            )
        }

        return KpiCalculationResult.Success(score = max)
    }
}
