package de.fraunhofer.iem.kpiCalculator.core.strategy

import de.fraunhofer.iem.kpiCalculator.model.kpi.hierarchy.KpiCalculationResult

object RatioKPICalculationStrategy {
    /**
     * Returns smallerValue / biggerValue, regardless in which order the values are given.
     */
    fun calculateKPI(childScores: List<Pair<KpiCalculationResult, Double>>): KpiCalculationResult {

        val successScores =
            childScores.map { it.first }.filterIsInstance<KpiCalculationResult.Success>()

        if (successScores.size != 2) {
            return KpiCalculationResult.Error(
                "Ratio calculation strategy called " +
                    "with ${successScores.size} valid elements, which is illegal."
            )
        }

        val firstValue = successScores.first().score
        val secondValue = successScores[1].score

        if (firstValue == secondValue && secondValue == 0) {
            return KpiCalculationResult.Success(0)
        }
        try {
            if (firstValue >= secondValue) {
                return KpiCalculationResult.Success(
                    score = (secondValue.toDouble() / firstValue.toDouble()).toInt() * 100
                )
            }
            return KpiCalculationResult.Success(
                score = ((firstValue.toDouble() / secondValue.toDouble()) * 100).toInt()
            )
        } catch (e: Exception) {
            return KpiCalculationResult.Error(e.message ?: e.toString())
        }
    }
}
